#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# This file profiles the provided binary on execution for the set platform.
# ------------------------------------------------------------------------------

# The BASEPROF is the PWD of profile.sh, regardless of the callers location.
export BASEPROF=$(cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd)

# Source helpers for colored prints.
source $BASEPROF/../helpers.sh


# Check if PLATFORM is valid.
function platformCheck {
    IFS="|"
    options=("lumi${IFS}mn5")

    if [[ -z "${PLATFORM}" ]]; then
        fatal "No platform selected. Options: [ lumi | mn5 ]"
        exit $?
    elif [[ ! "${IFS}${options[*]}${IFS}" =~ "${IFS}${PLATFORM}${IFS}" ]]; then
        fatal "Platform '${PLATFORM}' unknown. Options: [ lumi | mn5 ]"
        exit $?
    fi
}

# Profile binary on Lumi.
function profile_lumi {
    # Create folders for results.
    RESULTS=$BASEPROF/res-lumi-$1
    rm -rf $RESULTS
    mkdir -p $RESULTS/rocprof/

    # Set runtime variables.
    export OMP_NUM_THREADS=1
    export MPICH_GPU_SUPPORT_ENABLED=1
    ROCMFILES="$RESULTS/rocprof/rocprof.csv"

    # Profile the provided binary.
    info "[$(date +'%H:%M:%S')]  Profiling '$1'.."
    rocprof --hip-trace --roctx-trace -o $ROCMFILES $BASEPROF/../bin/$1 \
        > >(tee $RESULTS/out.log) 2> >(tee $RESULTS/err.log >&2)

    # Inform according to return value of rocprof.
    retval=$?
    if [ $retval -ne 0 ]; then
        fatal "[$(date +'%H:%M:%S')]  Failed profiling '$1' with error code $retval"
    else
        success "[$(date +'%H:%M:%S')]  Finished profiling '$1'"
    fi
}

# Profile binary on MareNostrum 5.
function profile_mn5 {
    # TODO: Implement.
    echo "To be created.."
}

# Entry function.
function main {
    # Check if platform is set correctly.
    platformCheck

    # Check if provided binary exists.
    command -v $BASEPROF/../bin/$1 >/dev/null @>&1 || \
        { fatal "Binary '$1' is not in bin folder."; exit $?; }

    # Load modules.
    source $BASEPROF/../load_modules.sh

    # Profile provided binary on set platform.
    if [ "$PLATFORM" == "lumi" ]; then
        profile_lumi $1
        # sbatch profile_lumi.sh $1
    elif [ "$PLATFORM" == "mn5" ]; then
        profile_mn5 $1
    fi
}

main "$@"
